package scphEditor;

import java.util.Enumeration;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * DOMBuilder builds a DOM tree of Work (typically for saving to files).
 */
public class DOMBuilder implements OperandVisitor {

  private Document doc;
  private Node cursor;

  public DOMBuilder(Work w) {
    try {
      doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
      Element root = doc.createElement("work");
      doc.appendChild(root);
      cursor = root;
      for (Enumeration c = w.getChildren(); c.hasMoreElements(); ) {
        Operand child = (Operand) c.nextElement();
        child.acceptVisitor(this);
      }
    } catch (ParserConfigurationException ex) {
      ex.printStackTrace();
    }
  }

  public Document getDocument() {
    return doc;
  }

  /* (non-Javadoc)
   * @see scphEditor.OperandVisitor#visitVariable(scphEditor.Variable)
   */
  public void visitVariable(Variable v) {
    visitIdentifier(v);
  }

  /* (non-Javadoc)
   * @see scphEditor.OperandVisitor#visitIdentifier(scphEditor.Identifier)
   */
  public void visitIdentifier(Identifier id) {
    Element e = doc.createElement("identifier");
    e.setAttribute("name", id.getName());
    cursor.appendChild(e);
  }

  /* (non-Javadoc)
   * @see scphEditor.OperandVisitor#visitExpression(scphEditor.Expression)
   */
  public void visitExpression(Expression e) {
    // add myself
    Element elem = doc.createElement("expression");
    elem.setAttribute("operator", e.getOperator().id());
    elem.setAttribute("size", String.valueOf(e.countChildren()));
    elem.setAttribute("paren", String.valueOf(e.getParen()));
    elem.setAttribute("split", String.valueOf(e.getSplit()));
    cursor.appendChild(elem);
    // do my children
    Node oldcursor = cursor;
    cursor = elem;
    for (Enumeration c = e.getChildren(); c.hasMoreElements(); ) {
      Operand child = (Operand) c.nextElement();
      child.acceptVisitor(this);
    }
    cursor = oldcursor;
  }

  /* (non-Javadoc)
   * @see scphEditor.OperandVisitor#visitExpressionNary(scphEditor.ExpressionNary)
   */
  public void visitExpressionNary(ExpressionNary e) {
    visitExpression(e);
  }

}
