package scphEditor;

/**
 * DefaultOperator provides a default implementation of the IOperator interface.
 * You provide the operator properties (arity etc.) to the constructor, and
 * these properties will be used by the query services.
 */
public class DefaultOperator implements IOperator {
  private String id;

  private IEmbedding format;

  private String[] proverName;

  private int arity;

  private int precedence;

  private int associative;

  private boolean commutative;

  public String toString() {
    return id + " " + arity + " " + precedence + " " + associative + " "
        + commutative + " " + proverName.toString();
  }

  /**
   * Constructs an DefaultOperator object with the given properties.
   * 
   * @param id
   *          the new unique internal ID of the operator
   * @param arityOf
   *          arity of the operator
   * @param precOf
   *          precedence of the operator
   * @param assocOf
   *          associativity of the operator
   * @param commOf
   *          commutativity of the operator
   * @param format
   *          the screen formatting spec for this operator
   * @param proverName
   *          the name(s) of this operator in the prover
   */
  public DefaultOperator(String id, int arityOf, int precOf, int assocOf,
      boolean commOf, IEmbedding format, String[] proverName) {
    this.id = id;
    this.format = format;
    this.proverName = proverName;
    arity = arityOf;
    precedence = precOf;
    associative = assocOf;
    commutative = commOf;
  }

  /**
   * Returns the precedence of this operator.
   */
  public int precedence() {
    return precedence;
  }

  /**
   * Returns the associativity of this operator.
   */
  public int associative() {
    return associative;
  }

  /**
   * Returns the commutativity of this operator.
   */
  public boolean commutative() {
    return commutative;
  }

  /**
   * Returns the arity of this operator.
   */
  public int arity() {
    return arity;
  }

  /**
   * @return Returns the format.
   */
  public IEmbedding format() {
    return format;
  }

  /**
   * @return Returns the internal unique id.
   */
  public String id() {
    return id;
  }

  /**
   * @return Returns the names on the prover.
   */
  public String[] proverName() {
    return proverName;
  }

  /**
   * @return Returns the first name on the prover.
   */
  public String proverName0() {
    return proverName[0];
  }
}