package scphEditor;


/**
 * An EmbeddingNary object encapsulates presentation styles, in AWT
 * terms, of an operator and its operands.  The operator is assumed to
 * be n-ary and infix - the indexing methods will compute indexes
 * accordingly.
 */

public class EmbeddingNary implements IEmbedding
{
  private String symbol;

  /**
   * Creates an EmbeddingNary object.  Requires a string containing
   * the operator symbol.
   *
   * @param symbolOf the operator symbol.
   */
  public EmbeddingNary(String symbolOf)
  {
    symbol = symbolOf;
  }

  /**
   * Gives a new instance of MLMLayout2, the preferred layout manager
   * here.
   *
   * @see MLMLayout2
   */
  public ExpressionLayoutManager getNewLayout()
  {
    return new MLMLayout2();
  }

  public String symbol(int componentIndex)
  {
    if (componentIndex < 0)
      throw new ArrayIndexOutOfBoundsException();

    // symbol occurs at odd indexes
    return (componentIndex & 1) == 1 ? symbol : null;
  }

  public int componentIndex(int operandIndex)
  {
    if (operandIndex < 0)
      throw new ArrayIndexOutOfBoundsException();

    // operands go to even indexes
    return 2 * operandIndex;
  }

  public int operandIndex(int componentIndex)
  {
    if (componentIndex < 0 || (componentIndex & 1) == 1)
      throw new ArrayIndexOutOfBoundsException();

    return componentIndex >> 1;
  }

  public int componentCount(int operandCount)
  {
    return operandCount == 0 ? 0 : operandCount * 2 - 1;
  }
}
