package scphEditor;

import java.util.Observable;

/**
 * This AWT component is the view class for ExpressionNary objects.
 * It displays the expression recursively.  It automatically updates
 * itself whenever the expression changes (child manipulations), by
 * way of the Observable-Observer mechanism.
 *
 * @see ExpressionNary
 */
public class ExpressionNaryView extends ExpressionView
                                        // implements Observer, OperandView
{
  protected ExpressionNary myClient;

  /**
   * Constructor.
   * @param exOf the ExpressionNary object to view.
   * @param c the control object (event listener) for me.
   */
  public ExpressionNaryView(ExpressionNary exOf, OperandControl c)
  {
    super(exOf, c);
    myClient = exOf;
  }

  public void update(Observable o, Object arg)
  {
    Expression.Replacement change = (Expression.Replacement) arg;
    if (change != null) {
      int atComponentIndex =
        myEmbedding.componentIndex(change.at) + (paren ? 1 : 0);

      if (change.deleteCount > 0) {
        // delete so many children and the operators inside them:
        // deleteCount * 2 - 1 of them
        int c = change.deleteCount * 2 - 1;
        for (int i = 0; i < c; i++) remove(atComponentIndex);
        // delete one more operator if not inserting later
        if (change.insertCount == 0) {
          if (change.at == 0) remove(atComponentIndex);
          else remove(atComponentIndex - 1);
        }
      }

      if (change.insertCount > 0) {
        OperandViewFactory f = new OperandViewFactory(myControl);
        // insert extra operator if not deleting earlier
        if (change.deleteCount == 0) {
          if (change.at == 0)
            add(new AtomView(myEmbedding.symbol(1)),
                ExpressionLayoutManager.OP,
                atComponentIndex);
          else
            add(new AtomView(myEmbedding.symbol(1)),
                ExpressionLayoutManager.OP,
                atComponentIndex - 1);
        }
        // insert the children and inside operators:
        // insertCount * 2 - 1 of them
        int c = change.insertCount * 2 - 1;
        for (int i = 0, ci = atComponentIndex, oi = change.at;
             i < c;
             i++, ci++) {
          if ((i & 1) == 0)     // i is even, operand
            add(f.make(myClient.getChild(oi++)), ci);
          else                  // i is odd, operator
            add(new AtomView(myEmbedding.symbol(1)),
                ExpressionLayoutManager.OP, ci);
        }
      }
    }
    parenthesize();
    ((ExpressionLayoutManager) getLayout()).setSplit(myClient.getSplit());
    // may need validation here
    repaint();
  }
}
