package scphEditor;

import java.awt.*;
import java.util.Observer;
import java.util.Observable;

/**
 * This AWT component is the view class for Identifier objects.  It
 * displays the identifier name.  It automatically updates itself
 * whenever the identifier name changes, by way of the
 * Observable-Observer mechanism.
 *
 * @see Identifier
 */
public class IdentifierView extends AtomView implements Observer, OperandView
{
  /**
   * My Identifier object.
   */
  private Identifier myClient;

  /**
   * Creates an IdentifierView object that views the given Identifier.
   *
   * @param idOf the Identifier object to view.
   * @param c the control object (event listener) for me.
   */
  public IdentifierView(Identifier idOf, OperandControl c)
  {
    super(idOf.getName());
    // should setFont() here

    addMouseListener(c);
    addKeyListener(c);

    myClient = idOf;
  }

  public void addNotify()
  {
    myClient.addObserver(this);
    super.addNotify();
  }

  public void removeNotify()
  {
    myClient.deleteObserver(this);
    super.removeNotify();
  }

  public Operand getOperand()
  {
    return myClient;
  }

  public OperandVector getOperandVector()
  {
    return myClient;
  }

  public int operandIndex(Component child)
  {
    return -1;                  // or throw exception?
  }

  public Component getOperandView(int opindex)
  {
    return null;                // or throw exception?
  }

  /**
   * Called by my Identifier object automagically when it changes its
   * name.  This method will then:
   * <ol>
   * <li>Check that the parameter o is indeed my Identifier object.
   *     If not, skip.
   * <li>Obtain the new identifier name and display it.
   * </ol>
   *
   * @param o my Identifier object (or any Observable)
   * @param arg ignored.
   * @see Identifier
   */
  public void update(Observable o, Object arg)
  {
    if (o == myClient) {
      setText(myClient.getName());
    }
  }
}
