package scphEditor;

import java.util.ArrayList;
import java.util.List;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Loader parses scphEditor files.
 */
public class Loader extends DefaultHandler {

  private Work work;
  private List cursorStack;

  public Loader() {
    work = new Work();
    cursorStack = new ArrayList();
    cursorStack.add(new Cursor(work));
  }

  public Work getWork() {
    return work;
  }

  public void startElement(String uri, String localName, String qName,
      Attributes attributes) throws SAXException {
    if ("identifier".equals(qName)) {
      Identifier operand = Identifier.lookup(attributes.getValue("name"));
      Cursor c = (Cursor) cursorStack.get(cursorStack.size() - 1);
      c.addChild(operand);
    } else if ("expression".equals(qName)) {
      IOperator operator = OperatorStore.getById(attributes.getValue("operator"));
      Expression operand;
      if (operator.arity() == IOperator.nARY) {
        int n = Integer.parseInt(attributes.getValue("size"));
        operand = new ExpressionNary(operator, n);
      } else {
        operand = new Expression(operator);
      }
      operand.setSplit(Boolean.parseBoolean(attributes.getValue("split")));
      Cursor c = (Cursor) cursorStack.get(cursorStack.size() - 1);
      c.addChild(operand);
      cursorStack.add(new Cursor(operand));
    }
  }

  public void endElement(String uri, String localName, String qName)
      throws SAXException {
    if ("expression".equals(qName)) {
      cursorStack.remove(cursorStack.size() - 1);
    }
  }

  private static class Cursor {
    public OperandVector v;
    public int n;
    public Cursor(OperandVector v) {
      this.v = v;
      this.n = 0;
    }
    public void addChild(Operand x) {
      if (v instanceof Work) {
        v.addChild(x, -1);
      } else {
        v.setChild(x, n);
      }
      n++;
    }
  }
}
