package scphEditor;

/**
 * This is the ``visitor'' interface, as in the Visitor pattern, for
 * visiting Operand trees.  Since the acceptVisitor() methods in the
 * Operand class does not traverse or recurse into children, you have
 * do the traversal yourself, but it is up to you how (but please
 * document).  All this interface specifies is just the names (entry
 * points) of the callbacks.
 *
 * @see Operand
 */
public interface OperandVisitor
{
  /**
   * Called by Variable.acceptVisitor().
   *
   * @param v the Variable object that calls me.
   * @see Variable#acceptVisitor
   */
  void visitVariable(Variable v);

  /**
   * Called by Identifier.acceptVisitor().
   *
   * @param id the Identifier object that calls me.
   * @see Identifier#acceptVisitor
   */
  void visitIdentifier(Identifier id);

  /**
   * Called by Expression.acceptVisitor().
   *
   * @param e the Expression object that calls me.
   * @see Expression#acceptVisitor
   */
  void visitExpression(Expression e);

  /**
   * Called by ExpressionNary.acceptVisitor().
   *
   * @param e the ExpressionNary object that calls me.
   * @see ExpressionNary#acceptVisitor
   */
  void visitExpressionNary(ExpressionNary e);
}
