package scphEditor;

import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.ViewPart;

/**
 * OperatorViewer is the view that displays and permits selection of operators.
 */
public class OperatorViewer extends ViewPart {
  private ListViewer viewer;

  /**
   * OperatorsContentProvider provides IOperator objects from OperatorStore to
   * the operator view.
   */
  private class OperatorsContentProvider implements IStructuredContentProvider {
    /**
     * Nothing to do.
     */
    public void inputChanged(Viewer v, Object oldInput, Object newInput) {
    }

    /**
     * Nothing to do.
     */
    public void dispose() {
    }

    /**
     * @param parent
     *          ignored
     * @return an array of all operators from OperatorStore.
     */
    public Object[] getElements(Object parent) {
      return OperatorStore.getOperators().toArray();
    }
  }

  /**
   * OperatorLabelProvider translates an IOperator object to a display string
   * for the operator view.
   */
  private class OperatorLabelProvider extends LabelProvider {
    /**
     * Translate an IOperator to a sample expression using that operator and
     * "blank" operands.
     * 
     * @param element
     *          the IOperator object
     * @return sample expression
     */
    public String getText(Object element) {
      IOperator op = (IOperator) element;
      IEmbedding format = op.format();
      int arity = op.arity();
      if (arity < 0)
        arity = 2;
      StringBuffer b = new StringBuffer();
      int n = format.componentCount(arity);
      for (int i = 0; i < n; ++i) {
        String s = format.symbol(i);
        b.append(s != null ? s : "_");
      }
      return b.toString();
    }
  }

  /**
   * The constructor.
   */
  public OperatorViewer() {
  }

  public void createPartControl(Composite parent) {
    setPartName("Operators");
    viewer = new ListViewer(parent, SWT.SINGLE | SWT.H_SCROLL | SWT.V_SCROLL);
    viewer.setContentProvider(new OperatorsContentProvider());
    viewer.setLabelProvider(new OperatorLabelProvider());
    viewer.setInput(OperatorStore.getOperators());
    viewer.addSelectionChangedListener(new ISelectionChangedListener() {
      public void selectionChanged(SelectionChangedEvent event) {
        if (ScphEditorContributor.recent != null) {
          IStructuredSelection s = (IStructuredSelection) event.getSelection();
          IOperator op = (IOperator) s.getFirstElement();
          Operand e;
          switch (op.arity()) {
          case IOperator.nARY:
            e = new ExpressionNary(op);
            break;
          case 0:
            e = (Identifier) op;
            break;
          default:
            e = new Expression(op);
          }
          ScphEditorContributor.recent.editor_widget.insert(e);
        }
      }
    });
  }

  /**
   * Pass the focus request to the viewer's control.
   */
  public void setFocus() {
    viewer.getControl().setFocus();
  }
}