package scphEditor;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.util.Observer;
import java.util.Observable;

// TODO move display to a separate view
public class ProverFrame extends Frame implements Observer, ActionListener
{
  private static final String proverName = "/home/trebla/scphEditor/interrupter hol -I /home/trebla/scphEditor /home/trebla/scphEditor/start.ML";
  private static final Font myFont = new Font("Monospaced", Font.PLAIN, 12);
  private Shell ps;
  private TextArea display;
  private TextField commandline;
  private StringBuffer output = new StringBuffer();
  private boolean ready = false;
  private PostExecute callback = null;

  private static ProverFrame singleton;

  public static abstract class PostExecute {
    public abstract void postExecute(String result);
  }

  public static ProverFrame instance()
  {
    if (singleton == null) singleton = new ProverFrame();
    return singleton;
  }

  private ProverFrame()
  {
    super("Prover Monitor");
    display = new TextArea();
    display.setEditable(false);
    display.setFont(myFont);
    commandline = new TextField();
    commandline.addActionListener(this);
    commandline.setFont(myFont);

    System.out.println("this is ProverFrame");
    try {
      ps = new Shell(proverName);
      ps.addObserver(this);
      ready = false;
      System.out.print("Please wait: loading a *huge* theorem prover... ");
      ps.start();
      synchronized (output) {
        while (!ready)
          try { output.wait(); } catch (InterruptedException ex) {}
      }
      System.out.println("done loading theorem prover");
    }
    catch (IOException e) {
      ps = null;
      display.setText("Unable to start prover\n");
      System.out.println("Unable to load theorem prover.  You can edit but not prove.");
      System.out.println(e);
    }
    commandline.setEditable(ps != null);

    add("Center", display);
    add("South", commandline);

    addWindowListener(new WindowAdapter() {
      public void windowClosed(WindowEvent e) {
        System.out.println("ProverFrame closed");
        if (ps != null) ps.kill();
      }
    });
  }

  public void actionPerformed(ActionEvent a) 
  {
    execute(commandline.getText());
    commandline.setText("");
  }

  public void update(Observable o, Object arg)
  {
    String cmd = (String)arg;
    display.append(cmd);
    synchronized (output) {
      if (cmd.equals("- ")) {
        ready = true;
        output.notify();
        if (callback != null) {
          callback.postExecute(output.toString());
          callback = null;
        }
      }
      else if (cmd.endsWith("\n- ")) {
        ready = true;
        output.append(cmd.substring(0, cmd.length() - 3));
        output.notify();
        if (callback != null) {
          callback.postExecute(output.toString());
          callback = null;
        }
      }
      else {
        output.append(cmd);
      }
    }
  }

  public String execute(String cmd)
  {
    if (ps == null) return "";
    display.append(cmd + "\n");
    output.setLength(0);
    ready = false;
    try {
      ps.execute(cmd);
    }
    catch (IOException e) {
      display.append("Lost connection to prover\n");
      return "";
    }
    synchronized (output) {
      try {
        while (!ready) output.wait();
      }
      catch (InterruptedException e) {}
      return output.toString();
    }
  }

  public String execute(String cmd, PostExecute p)
  {
    if (ps == null) {
      p.postExecute("");
      return null;
    }
    display.append(cmd + "\n");
    output.setLength(0);
    ready = false;
    callback = p;
    try {
      ps.execute(cmd);
    }
    catch (IOException e) {
      display.append("Lost connection to prover\n");
      p.postExecute("");
      callback = null;
      return null;
    }
    return null;
  }

  public void interrupt() {
    if (ps != null) ps.interrupt();
  }

  public static void kill()
  {
    if (singleton.ps != null) {
      try {
        singleton.ps.execute("quit();");
      }
      catch (IOException ex) {
      }
      // if (singleton.ps != null) ps.kill();
    }
  }
}
