package scphEditor;

import java.awt.Frame;
import java.awt.ScrollPane;
import java.io.IOException;
import java.io.InputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.awt.SWT_AWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;
import org.xml.sax.SAXException;

/**
 * ScphEditorPart defines our refinement/proof editor.
 */
public class ScphEditorPart extends EditorPart {

  /**
   * Reference to the widget/control that displays/edits proof content.
   */
  Workspace editor_widget;

  /**
   * Reference to the proof content.
   */
  Work proof_content;

  /**
   * Oh boy am I dirty.
   */
  boolean dirty = false;

  public ScphEditorPart() {
    super();
  }

  public void doSave(IProgressMonitor monitor) {
    try {
      final DOMBuilder b = new DOMBuilder(proof_content);
      final Transformer t = TransformerFactory.newInstance().newTransformer();
      final PipedOutputStream push = new PipedOutputStream();
      PipedInputStream pull = new PipedInputStream(push);
      new Thread(new Runnable() {
        public void run() {
          try {
            t.transform(new DOMSource(b.getDocument()), new StreamResult(push));
          } catch (TransformerException e) {
            e.printStackTrace();
          }
          try {
            push.close();
          } catch (IOException e1) {
            e1.printStackTrace();
          }
        }
      }).start();
      ((IFileEditorInput) getEditorInput()).getFile().setContents(pull, true, true, monitor);
      // setContents will close pull
    } catch (TransformerConfigurationException e) {
      e.printStackTrace();
    } catch (TransformerFactoryConfigurationError e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    } catch (CoreException e) {
      e.printStackTrace();
    }
    dirty = false;
    firePropertyChange(PROP_DIRTY);
  }

  public void doSaveAs() {
    // TODO Auto-generated method stub

  }

  private void doLoad() {
    try {
      Loader loader = new Loader();
      SAXParser p = SAXParserFactory.newInstance().newSAXParser();
      InputStream in = ((IFileEditorInput) getEditorInput()).getFile()
          .getContents();
      p.parse(in, loader);
      in.close();
      proof_content = loader.getWork();
    } catch (ParserConfigurationException ex) {
      ex.printStackTrace();
    } catch (SAXException ex) {
      ex.printStackTrace();
    } catch (CoreException ex) {
      ex.printStackTrace();
    } catch (IOException ex) {
      ex.printStackTrace();
    }
  }

  public void init(IEditorSite site, IEditorInput input)
      throws PartInitException {
    if (!(input instanceof IFileEditorInput)) {
      throw new PartInitException("Not a file.");
    }
    setSite(site);
    setInput(input);
  }

  public boolean isDirty() {
    return dirty;
  }

  public boolean isSaveAsAllowed() {
    return false;
  }

  public void createPartControl(Composite parent) {
    make_editor_widget(parent);
    parent.setLayout(new FillLayout());
    IEditorInput input = getEditorInput();
    setPartName(input.getName());
    setTitleToolTip(input.getToolTipText());
    ProverShell.instance();
  }

  Composite stub;
  Frame awtFrame;

  private void make_editor_widget(Composite parent) {
    stub = new Composite(parent, SWT.EMBEDDED);
    awtFrame = SWT_AWT.new_Frame(stub);
    ScrollPane sp = new ScrollPane(ScrollPane.SCROLLBARS_ALWAYS);
    awtFrame.add(sp);
    doLoad();
    // proof_content = new Work();
    editor_widget = new Workspace(proof_content, this);
    sp.add(editor_widget);
    awtFrame.pack();
  }

  public void setFocus() {
    stub.setFocus();
    //awtFrame.requestFocus();
    editor_widget.requestFocus();
  }


  public void dispose() {
    super.dispose();
    if (ScphEditorContributor.recent == this) {
      ScphEditorContributor.recent = null;
    }
  }

  public void markDirty() {
    this.stub.getDisplay().asyncExec(new Runnable() {
      public void run() {
        if (!dirty) {
          dirty = true;
          firePropertyChange(PROP_DIRTY);
        }
      }
    });
  }

}