package scphEditor;

import java.util.Enumeration;
import java.util.Stack;
import java.util.StringTokenizer;

public class TermBuilder implements OperandVisitor
{
  private StringBuffer buffer;

  public TermBuilder()
  {
    buffer = new StringBuffer();
  }

  public static Operand make(String s)
  {
    StringTokenizer tokens = new StringTokenizer(s);
    Stack stack = new Stack();

    while (tokens.hasMoreTokens()) {
      String t = tokens.nextToken();
      if ("begin".equals(t)) break;
    }

    while (tokens.hasMoreTokens()) {
      String t = tokens.nextToken();
      if (t.equals("\\")) {
      }
      else if (t.equals(".")) {
        Operand parent = (Operand) stack.pop();
        Operand child = (Operand) stack.pop();
        int i = 0;
        while (parent.getChild(i) != null) i++;
        parent.setChild(child, i);
        stack.push(parent);
      }
      else {
        Operand x = null;
        IOperator o = OperatorStore.getByProverName(t);
        if (o == null) {
          x = Identifier.lookup(t);
        } else {
          if (o.arity() == IOperator.nARY) {
            x = new ExpressionNary(o);
          }
          else {
            x = new Expression(o);
          }
        }
        stack.push(x);
      }
    }
    return stack.empty() ? null : (Operand) stack.pop();
  }

  public String make(Operand p)
  {
    buffer.setLength(0);
    if (p == null) {
      // hmm...
    }
    else {
      p.acceptVisitor(this);
    }
    return buffer.toString();
  }

  public void visitVariable(Variable v)
  {
    buffer.append(v.getPName());
    buffer.append(" : ");
    buffer.append(v.getType());
  }

  public void visitIdentifier(Identifier id)
  {
    buffer.append(id.getPName());
  }

  public void visitExpression(Expression e)
  {
    buffer.append("$");
    buffer.append(e.getOperator().proverName0());
    buffer.append(" ");
    Enumeration i = e.getChildren();
    while (i.hasMoreElements()) {
      Operand p = (Operand) i.nextElement();
      buffer.append("(");
      p.acceptVisitor(this);
      buffer.append(") ");
    }
  }

  public void visitExpressionNary(ExpressionNary e)
  {
    String opname = e.getOperator().proverName0();
    Enumeration i;
    Operand p;

    switch (e.getOperator().associative()) {
    case IOperator.LEFT:
      for (int j = 1; j < e.countChildren(); j++) {
        buffer.append("(");
      }
      i = e.getChildren();
      p = (Operand) i.nextElement();
      buffer.append("(");
      p.acceptVisitor(this);
      buffer.append(")");
      while (i.hasMoreElements()) {
        p = (Operand) i.nextElement();
        buffer.append(" ");
        buffer.append(opname);
        buffer.append(" (");
        p.acceptVisitor(this);
        buffer.append("))");
      }
      break;
    case IOperator.RIGHT:
    case IOperator.ASSOC:
      i = e.getChildren();
      p = (Operand) i.nextElement();
      buffer.append("(");
      p.acceptVisitor(this);
      buffer.append(") ");
      while (i.hasMoreElements()) {
        p = (Operand) i.nextElement();
        buffer.append(" ");
        buffer.append(opname);
        buffer.append(" (");
        p.acceptVisitor(this);
        buffer.append(") ");
      }
      break;
    }
  }
}
