package scphEditor;

import java.util.Vector;
import java.util.Enumeration;

/**
 * This class models an scphEditor document, a list of operands and
 * stuff.
 *
 * <p>When a Work object changes, it will call notifyOberservers()
 * with an Integer object as a parameter:
 *
 * <ul>
 * <li>If the value is n, n>=1, then new stuff is inserted at index n-1.
 * <li>If the value is -n, n>=1, then old stuff at index n-1 is deleted.
 * </ul>
 */
public class Work extends OperandVector
{
  /**
   * List of operands and stuff.
   */
  private Vector content;

  /**
   * Constructor.
   */
  public Work()
  {
    content = new Vector();
  }

  public void setChild(Operand newChild, int atIndex)
  {
//     content.setElementAt(newChild, atIndex);
//     setChanged();
//     notifyObservers(new Integer(atIndex+1));
    delChild(atIndex);
    addChild(newChild, atIndex);
  }

  public void addChild(Operand newChild, int atIndex)
  {
    if (atIndex == -1) atIndex = content.size();
    content.insertElementAt(newChild, atIndex);
    setChanged();
    notifyObservers(new Integer(atIndex+1));
  }

  public Operand delChild(int atIndex)
  {
    Operand o = (Operand) content.elementAt(atIndex);
    content.removeElementAt(atIndex);
    setChanged();
    notifyObservers(new Integer(-atIndex-1));
    return o;
  }

  public Operand getChild(int atIndex)
  {
    return (Operand) content.elementAt(atIndex);
  }

  public Enumeration getChildren()
  {
    return content.elements();
  }

  public int countChildren()
  {
    return content.size();
  }
}
