package scphEditor;

import java.awt.*;
import java.awt.event.*;
import java.util.Observer;
import java.util.Observable;
import java.util.Stack;

// Workspace holds the formulae

public class Workspace extends Panel implements Observer, OperandVectorView {
  public static Font IDFONT = new Font("Serif", Font.BOLD, 14);

  public static Font OPFONT = new Font("Serif", Font.BOLD, 14);

  public static Font UNDEFFONT = new Font("Serif", Font.BOLD, 14);

  public static Font TEXTFONT = OPFONT;

  private Work myClient;

  private OperandControl childControl;

  private ScphEditorPart master;

  public Workspace(Work w, ScphEditorPart p) {
    super();

    master = p;

    myClient = w;
    myClient.addObserver(this);
    childControl = new EditorOperandControl();

    setLayout(new RowLayout(8));
    setFont(TEXTFONT);
    setForeground(Color.black);
    setBackground(Color.white);

    WorkspaceControl wc = new WorkspaceControl();
    addMouseListener(wc);
    addKeyListener(wc);

    OperandViewFactory f = new OperandViewFactory(childControl);
    for (int i = 0; i < myClient.countChildren(); i++) {
      add(f.make(myClient.getChild(i)));
    }
  }

  public Work getWork() {
    return myClient;
  }

  public OperandVector getOperandVector() {
    return myClient;
  }

  public int operandIndex(Component c) {
    Component[] cs = getComponents();
    for (int i = 0; i < cs.length; i++) {
      if (cs[i] == c)
        return i;
    }
    return -1; // or throw exception?
  }

  public Component getOperandView(int opindex) {
    return getComponent(opindex);
  }

  public void update(Observable w, Object a) {
    if (myClient != w)
      return;
    int n = ((Integer) a).intValue();
    if (n >= 1) {
      OperandViewFactory f = new OperandViewFactory(childControl);
      add(f.make(myClient.getChild(n - 1)), n - 1);
      getParent().validate();
    } else { // n <= -1
      remove(-n - 1);
      getParent().validate();
    }
  }

  // double-buffering code to eliminate flickering
  // shamelessly stolen from the AWT website :)
  // Of course I understand how it works and I am not just pasting :) :)

  private Image offscreen = null;

  public void invalidate() {
    super.invalidate();
    offscreen = null;
    master.markDirty();
  }

  public void update(Graphics g) {
    paint(g);
  }

  public void paint(Graphics g) {
    Dimension size = getSize();
    if (offscreen == null) {
      offscreen = createImage(size.width, size.height);
    }
    Graphics og = offscreen.getGraphics();
    og.setClip(0, 0, size.width, size.height);
    super.paint(og);
    g.drawImage(offscreen, 0, 0, null);
    og.dispose();
  }

  Component[] selected = new Component[2];

  // inserts, at the selection point or the end, a new expression
  // with the specified operator.
  // may belong to Expression
  public void insert(Operand o) {
    if (selected[0] == null) { // then easy
      OperandVector target = getOperandVector();
      target.addChild(o, -1);
      Component c = getOperandView(target.countChildren() - 1);
      selectAfterInsert(c);
    } else {
      OperandContext context = new OperandContext(selected[0]);
      // could deselect() here but theoretically unnecessary
      context.parent.setChild(o, context.index);
      if (o.countChildren() > 0 && o.getChild(0) == null) {
        o.setChild(context.self, 0);
      }
      // ok, ugly to resort to instanceof, but ...
      if (context.parent instanceof ExpressionNary) {
        ((ExpressionNary) context.parent).ungroup(context.index);
      }
      getParent().validate();
      repaint();
      selectAfterInsert(((OperandVectorView) context.parentview)
          .getOperandView(context.index));
    }
  }

  // ungroup at the selection
  public void ungroup() {
    if (selected[0] == null)
      return;
    OperandContext context = new OperandContext(selected[0]);
    if (context.self != null)
      context.self.removeParen(context.parent);
    if (context.parent instanceof ExpressionNary) {
      ((ExpressionNary) context.parent).ungroup(context.index);
      deselect(0);
    }
    getParent().validate();
  }

  public void group() {
    OperandContext context = null, context2 = null;
    if (selected[0] == null)
      return;
    context = new OperandContext(selected[0]);
    if (selected[1] != null) {
      context2 = new OperandContext(selected[1]);
    }
    if (context.parent instanceof ExpressionNary && context2 != null
        && context.parent == context2.parent && context.index < context2.index) {
      // ok, finally :)
      ((ExpressionNary) context.parent).group(context.index, context2.index
          - context.index + 1);
      deselect(0);
      deselect(1);
    } else {
      if (context.self != null)
        context.self.addParen();
    }
    getParent().validate();
  }

  // delete the operand at the selection
  public void delete() {
    if (selected[0] != null) {
      OperandContext context = new OperandContext(selected[0]);
      if (context.self == null) {
        context.parent.delChild(context.index);
        if (context.parent instanceof ExpressionNary
            && context.parent.countChildren() == 1) {
          context = new OperandContext(context.parentview);
          context.parent.setChild(context.self.getChild(0), context.index);
        }
      } else {
        context.parent.setChild(null, context.index);
      }
      getParent().validate();
      repaint();
      int i = Math.min(context.index, context.parent.countChildren() - 1);
      if (i >= 0) {
        select(0, ((OperandVectorView) context.parentview).getOperandView(i));
      } else {
        deselect(0);
      }
    }
  }

  public void split(boolean s) {
    if (selected[0] == null)
      return;
    OperandContext context = new OperandContext(selected[0]);
    if (context.self == null)
      return;
    context.self.setSplit(s);
    context.selfview.invalidate();
    getParent().validate();
  }

  public void copy() {
    if (selected[0] == null || selected[1] == null)
      return;
    OperandContext src, dst;
    src = new OperandContext(selected[1]);
    dst = new OperandContext(selected[0]);
    deselect(0);
    deselect(1);
    if (src.self == null) {
      dst.parent.setChild(null, dst.index);
    } else {
      dst.parent.setChild((Operand) src.self.clone(), dst.index);
    }
    getParent().validate();
  }

  public void swap() {
    if (selected[0] == null || selected[1] == null)
      return;
    if (selected[0] == selected[1])
      return;
    OperandContext s, t;
    s = new OperandContext(selected[0]);
    t = new OperandContext(selected[1]);
    if (s.self == t.self)
      return;
    Operand s_ = t.self, t_ = s.self;
    s.parent.setChild(s_, s.index);
    t.parent.setChild(t_, t.index);
    deselect(0);
    deselect(1);
    getParent().validate();
  }

  // c is recently inserted
  private void selectAfterInsert(Component nov) {
    // look for root of tree
    Stack path = new Stack();
    Component r = nov;
    while (r.getParent() != null && r.getParent() instanceof OperandView) {
      path.push(r);
      r = r.getParent();
    }
    // post-condition: r is root, path is path of view objects leading
    // from r to nov, not including r (even if r == nov)

    // select first null child or nov itself
    Component firstnull = findFirstNull(r, path);
    if (firstnull != null)
      select(0, firstnull);
    else
      select(0, nov);
  }

  // destroys the path along the way
  private Component findFirstNull(Component root, Stack path) {
    if (path.empty() && root instanceof NullView)
      return root;
    OperandVectorView r = (OperandVectorView) root;
    OperandVector op = r.getOperandVector();
    int start = path.empty() ? 0 : r.operandIndex((Component) path.pop());
    for (int i = start; i < op.countChildren(); i++) {
      Component c = findFirstNull(r.getOperandView(i), path);
      if (c != null)
        return c;
    }
    return null;
  }

  public void deselect(int i) {
    if (selected[i] != null) {
      selected[i].setForeground(null);
      selected[i].setBackground(null);
      selected[i].repaint();
      selected[i] = null;
    }
    requestFocus();
  }

  private static final Color[] colour = { Color.black, Color.blue };

  public void select(int i, Component c) {
    if (selected[i] != null) {
      selected[i].setForeground(null);
      selected[i].setBackground(null);
      selected[i].repaint();
      selected[i] = null;
    }
    c.setForeground(Color.white);
    c.setBackground(colour[i]);
    c.repaint();
    selected[i] = c;
    if (i == 0) c.requestFocus();
  }

  public Component getSelection() {
    return selected[0];
  }

  class EditorOperandControl extends OperandControl {
    /**
     * Left or right click: selects the operand, and makes the owning Editor the
     * most-recent one.
     */
    public void mouseClicked(MouseEvent e) {
      if ((e.getModifiers() & InputEvent.BUTTON1_MASK) != 0) {
        if (selected[0] == e.getComponent()) {
          deselect(0);
        } else {
          select(0, e.getComponent());
        }
        // recentFocus = Editor.this;
      } else if ((e.getModifiers() & InputEvent.BUTTON3_MASK) != 0) {
        if (selected[1] == e.getComponent()) {
          deselect(1);
        } else {
          select(1, e.getComponent());
        }
        // recentFocus = Editor.this;
      }
    }

    public void keyPressed(KeyEvent e) {
      int key = e.getKeyCode();
      OperandContext context = new OperandContext(e.getComponent());

      switch (key) {
      /*
       * cursor keys
       */
      case KeyEvent.VK_LEFT:
        if (context.index == 0)
          break;
        select(0, ((OperandVectorView) context.parentview)
            .getOperandView(context.index - 1));
        break;
      case KeyEvent.VK_RIGHT:
        if (context.index == context.parent.countChildren() - 1)
          break;
        select(0, ((OperandVectorView) context.parentview)
            .getOperandView(context.index + 1));
        break;
      case KeyEvent.VK_UP:
        if (!(context.parent instanceof Operand))
          break;
        select(0, context.parentview);
        break;
      case KeyEvent.VK_DOWN:
        if (context.self == null || context.self.countChildren() == 0)
          break;
        select(0, ((OperandView) context.selfview).getOperandView(0));
        break;

      /*
       * editing keys
       */
      case KeyEvent.VK_DELETE:
        delete();
        break;
      }
    }

    public void keyTyped(KeyEvent e) {
      char key = e.getKeyChar();
      OperandContext context = new OperandContext(e.getComponent());
      if (Character.isLetterOrDigit(key) || key == '\'') {
        if (context.self == null) {
          context.parent.setChild(Identifier.lookup(String.valueOf(key)), context.index);
          deselect(0);  // should select new identifier
          getParent().validate();
        } else if (context.self instanceof Identifier) {
          Identifier i = (Identifier) context.self;
          context.parent.setChild(Identifier.lookup(i.getName() + key), context.index);
          deselect(0);  // should select new identifier
          getParent().validate();
        }
      }
    }
  }

  //   public static Workspace getWorkspace(Component c)
  //   {
  //     for (Component i = c; i != null; i = i.getParent()) {
  //       if (i instanceof Workspace)
  //         return (Workspace) i;
  //     }
  //     return null;
  //   }
}