package scphEditor.test;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;

import scphEditor.IOperator;
import scphEditor.OperatorStore;

import junit.framework.TestCase;

/**
 * OperatorStoreTest tests scphEditor.OperatorStore.parse.
 */
public class OperatorStoreTest extends TestCase {

  /**
   *  
   */
  public OperatorStoreTest() {
    super();
  }

  /**
   * Constructor for OperatorStoreTest.
   * 
   * @param name
   */
  public OperatorStoreTest(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    super.setUp();
    OperatorStore.clear();
  }

  protected void tearDown() throws Exception {
    OperatorStore.clear();
    super.tearDown();
  }

  public void testParse0() throws Exception {
    String teststring = "PLUS -2 4 ASSOC true \"+\" \"+\" \"int_add\"\n"
        + "TIMES -2 3 ASSOC true \"\" \"*\" \"int_mult\"\n";
    Reader r = new StringReader(teststring);
    OperatorStore.parse(r);
    IOperator op;

    op = OperatorStore.getById("PLUS");
    assertEquals("PLUS", op.id());
    assertEquals(-2, op.arity());
    assertEquals(4, op.precedence());
    assertEquals(IOperator.ASSOC, op.associative());
    assertEquals(true, op.commutative());
    assertNotNull("format", op.format());
    assertEquals("+", op.format().symbol(1));
    assertEquals(2, op.proverName().length);
    assertEquals("+", op.proverName()[0]);
    assertEquals("int_add", op.proverName()[1]);

    op = OperatorStore.getById("TIMES");
    assertEquals("TIMES", op.id());
    assertEquals(-2, op.arity());
    assertEquals(3, op.precedence());
    assertEquals(IOperator.ASSOC, op.associative());
    assertEquals(true, op.commutative());
    assertNotNull("format", op.format());
    assertEquals("", op.format().symbol(1));
    assertEquals(2, op.proverName().length);
    assertEquals("*", op.proverName()[0]);
    assertEquals("int_mult", op.proverName()[1]);
  }

  public void testParse1() throws Exception {
    String teststring = "AND -2 9 ASSOC true \"\u2227\" \"/\\\\\"\n";
    InputStream s = new ByteArrayInputStream(teststring.getBytes("UTF-8"));
    OperatorStore.parse(s);
    IOperator op;

    op = OperatorStore.getById("AND");
    assertEquals("AND", op.id());
    assertEquals(-2, op.arity());
    assertEquals(9, op.precedence());
    assertEquals(IOperator.ASSOC, op.associative());
    assertEquals(true, op.commutative());
    assertNotNull("format", op.format());
    assertEquals("\u2227", op.format().symbol(1));
    assertEquals(1, op.proverName().length);
    assertEquals("/\\", op.proverName()[0]);
  }

  public void testParseFile() throws Exception {
    OperatorStore.parse("/home/trebla/workspace/scphEditor-Plugin/operators");
    IOperator op;
    
    op = OperatorStore.getById("PLUS");
    assertEquals("PLUS", op.id());
    assertEquals(-2, op.arity());
    assertEquals(4, op.precedence());
    assertEquals(IOperator.ASSOC, op.associative());
    assertEquals(true, op.commutative());
    assertNotNull("format", op.format());
    assertEquals("+", op.format().symbol(1));
    assertEquals(2, op.proverName().length);
    assertEquals("+", op.proverName()[0]);
    assertEquals("int_add", op.proverName()[1]);

    op = OperatorStore.getById("TIMES");
    assertEquals("TIMES", op.id());
    assertEquals(-2, op.arity());
    assertEquals(3, op.precedence());
    assertEquals(IOperator.ASSOC, op.associative());
    assertEquals(true, op.commutative());
    assertNotNull("format", op.format());
    assertEquals("", op.format().symbol(1));
    assertEquals(2, op.proverName().length);
    assertEquals("*", op.proverName()[0]);
    assertEquals("int_mult", op.proverName()[1]);

    op = OperatorStore.getById("AND");
    assertEquals("AND", op.id());
    assertEquals(-2, op.arity());
    assertEquals(9, op.precedence());
    assertEquals(IOperator.ASSOC, op.associative());
    assertEquals(true, op.commutative());
    assertNotNull("format", op.format());
    assertEquals("\u2227", op.format().symbol(1));
    assertEquals(1, op.proverName().length);
    assertEquals("/\\", op.proverName()[0]);
  }
}
